-- Libraries

local L = AceLibrary("AceLocale-2.2"):new("GridLayoutPlus")
local RL = AceLibrary("Roster-2.1")

-- New Grid module

GridLayoutPlus = Grid:NewModule("GridLayoutPlus", "AceConsole-2.0")

GridLayoutPlus.defaultDB = {
	autoLayout = true,
	withPets = true,
	maxColumns = 16,
	showWarnings = true,
}


-- Init

function GridLayoutPlus:OnEnable()
	local options = GridLayout.options.args
	options["autoLayout"] = {
		type = "toggle",
		name = L["Auto adjust layout (+)"],
		desc = L["Automatically adjust layout to fit raid size"],
		get = function ()
			return GridLayout.db.profile.autoLayout
		end,
		set = function (v)
			GridLayout.db.profile.autoLayout = v
			GridLayoutPlus:RosterLib_RosterUpdated()
		end,	
	}
	options["withPets"] = {
		type = "toggle",
		name = L["Use pet layouts (+)"],
		desc = L["Use pet layouts when auto adjusting (if possible)"],
		get = function ()
			return GridLayout.db.profile.withPets
		end,
		set = function (v)
			GridLayout.db.profile.withPets = v
			GridLayoutPlus:RosterLib_RosterUpdated()
		end,	
	}
	options["maxColumns"] = {
		type = "range",
		name = L["Max number of columns (+)"],
		desc = L["Maximum number of columns when auto-adjusting (WARNING: May hide units)"],
		min = 1,
		max = 16,
		step = 1,
		get = function ()
			return GridLayout.db.profile.maxColumns
		end,
		set = function (v)
			GridLayout.db.profile.maxColumns = v
			GridLayoutPlus:RosterLib_RosterUpdated()
		end,	
	}
	options["showWarnings"] = {
		type = "toggle",
		name = L["Show warnings (+)"],
		desc = L["Show warnings when units are being hidden"],
		get = function ()
			return GridLayout.db.profile.showWarnings
		end,
		set = function (v)
			GridLayout.db.profile.showWarnings = v
			GridLayoutPlus:RosterLib_RosterUpdated()
		end,	
	}
	self:RegisterEvent("RosterLib_RosterUpdated")
end


-- Additional Layouts

GridLayout:AddLayout(L["By Group 40 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		groupFilter = "4",
	},
	[5] = {
		groupFilter = "5",
	},
	[6] = {
		groupFilter = "6",
	},
	[7] = {
		groupFilter = "7",
	},
	[8] = {
		groupFilter = "8",
	},
	[9] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 35 (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		groupFilter = "4",
	},
	[5] = {
		groupFilter = "5",
	},
	[6] = {
		groupFilter = "6",
	},
	[7] = {
		groupFilter = "7",
	},
})

GridLayout:AddLayout(L["By Group 35 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		groupFilter = "4",
	},
	[5] = {
		groupFilter = "5",
	},
	[6] = {
		groupFilter = "6",
	},
	[7] = {
		groupFilter = "7",
	},
	[8] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 30 (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		groupFilter = "4",
	},
	[5] = {
		groupFilter = "5",
	},
	[6] = {
		groupFilter = "6",
	},
})

GridLayout:AddLayout(L["By Group 30 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		groupFilter = "4",
	},
	[5] = {
		groupFilter = "5",
	},
	[6] = {
		groupFilter = "6",
	},
	[7] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 20 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		groupFilter = "4",
	},
	[5] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 15 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 5 (+)"], {
	[1] = {
		groupFilter = "1",
	},
})

GridLayout:AddLayout(L["By Group 5 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Sorted Class (+)"], {
	[1] = {
		groupFilter = "WARRIOR,ROGUE,DRUID,PALADIN,SHAMAN,PRIEST,MAGE,WARLOCK,HUNTER",
		groupBy = "CLASS",
		groupingOrder = "WARRIOR,ROGUE,DRUID,PALADIN,SHAMAN,PRIEST,MAGE,WARLOCK,HUNTER",
		unitsPerColumn = 5,
		maxColumns = 8,
	},
})

GridLayout:AddLayout(L["By Sorted Class w/Pets (+)"], {
	[1] = {
		groupFilter = "WARRIOR,ROGUE,DRUID,PALADIN,SHAMAN,PRIEST,MAGE,WARLOCK,HUNTER",
		groupBy = "CLASS",
		groupingOrder = "WARRIOR,ROGUE,DRUID,PALADIN,SHAMAN,PRIEST,MAGE,WARLOCK,HUNTER",
		unitsPerColumn = 5,
		maxColumns = 8,
	},
	[2] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		groupBy = "CLASS",
		groupingOrder = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	}
})


-- Event handlers

function GridLayoutPlus:RosterLib_RosterUpdated()
	if GridLayout.db.profile.autoSizeFrames and (GetNumRaidMembers() > 0) then
		local maxsubgroup = 0
		local newlayout = nil
		local newmaxraidmembers = 40
		local newwithpets = true
		for u in RL:IterateRoster(nil) do
			if u.subgroup > maxsubgroup then
				maxsubgroup = u.subgroup
			end
		end
		if string.find(GridLayout.db.profile.layout, L["By Group"]) then
			if (maxsubgroup > 7) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > 15) then
				newlayout = L["By Group 40 w/Pets (+)"]
			elseif (maxsubgroup > 7) and (GridLayout.db.profile.maxColumns > 7) then
				newlayout = L["By Group 40"]
				newwithpets = nil
			elseif (maxsubgroup > 6) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > 13) then
				newlayout = L["By Group 35 w/Pets (+)"]
				newmaxraidmembers = 35
			elseif (maxsubgroup > 6) and (GridLayout.db.profile.maxColumns > 6) then
				newlayout = L["By Group 35 (+)"]
				newmaxraidmembers = 35
				newwithpets = nil
			elseif (maxsubgroup > 5) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > 11) then
				newlayout = L["By Group 30 w/Pets (+)"]
				newmaxraidmembers = 30
			elseif (maxsubgroup > 5) and (GridLayout.db.profile.maxColumns > 5) then
				newlayout = L["By Group 30 (+)"]
				newmaxraidmembers = 30
				newwithpets = nil
			elseif (maxsubgroup > 4) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > 9) then
				newlayout = L["By Group 25 w/Pets"]
				newmaxraidmembers = 25
			elseif (maxsubgroup > 4) and (GridLayout.db.profile.maxColumns > 4) then
				newlayout = L["By Group 25"]
				newmaxraidmembers = 25
				newwithpets = nil
			elseif (maxsubgroup > 3) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > 7) then
				newlayout = L["By Group 20 w/Pets (+)"]
				newmaxraidmembers = 20
			elseif (maxsubgroup > 3) and (GridLayout.db.profile.maxColumns > 3) then
				newlayout = L["By Group 20"]
				newmaxraidmembers = 20
				newwithpets = nil
			elseif (maxsubgroup > 2) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > 5) then
				newlayout = L["By Group 15 w/Pets (+)"]
				newmaxraidmembers = 15
			elseif (maxsubgroup > 1) and (GridLayout.db.profile.maxColumns > 2) then
				newlayout = L["By Group 15"]
				newmaxraidmembers = 15
				newwithpets = nil
			elseif (maxsubgroup > 1) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > 3) then
				newlayout = L["By Group 10 w/Pets"]
				newmaxraidmembers = 10
			elseif (maxsubgroup > 1) and (GridLayout.db.profile.maxColumns > 1) then
				newlayout = L["By Group 10"]
				newmaxraidmembers = 10
				newwithpets = nil
			elseif GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > 1) then
				newlayout = L["By Group 5 w/Pets (+)"]
				newmaxraidmembers = 5
			else
				newlayout = L["By Group 5 (+)"]
				newmaxraidmembers = 5
				newwithpets = nil
			end
		elseif string.find(GridLayout.db.profile.layout, L["By Sorted Class"]) then
			if GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > 15) then
				newlayout = L["By Sorted Class w/Pets (+)"]
			else
				newlayout = L["By Sorted Class (+)"]
				newwithpets = nil
			end
		end
		if newlayout and (GridLayout.db.profile.layout ~= newlayout) then
			GridLayout.db.profile.layout = newlayout
			GridLayout:LoadLayout(newlayout)
			if GridLayout.db.profile.showWarnings and GridLayout.db.profile.withPets and (not newwithpets) then
				self:Print(L["Your settings are causing pet units to be hidden"])
			end
			if GridLayout.db.profile.showWarnings and (GetNumRaidMembers() > newmaxraidmembers) then
				self:Print(L["Your settings are causing player units to be hidden"])
			end
		end
	end
end
